/*
 * $Id: break.c,v 1.4 2000/01/07 07:55:00 lculik Exp $
 */

/*
 * Harbour Project source code:
 * BREAK() function
 *
 * Copyright 1999 {list of individual authors and e-mail addresses}
 * www - http://www.harbour-project.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version, with one exception:
 *
 * The exception is that if you link the Harbour Runtime Library (HRL)
 * and/or the Harbour Virtual Machine (HVM) with other files to produce
 * an executable, this does not by itself cause the resulting executable
 * to be covered by the GNU General Public License. Your use of that
 * executable is in no way restricted on account of linking the HRL
 * and/or HVM code into it.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA (or visit
 * their web site at http://www.gnu.org/).
 *
 */

#include "extend.h"
#include "ctoharb.h"

/*  $DOC$
 *  $FUNCNAME$
 *      BREAK()
 *  $CATEGORY$
 *      Error recovery
 *  $ONELINER$
 *      Exits from a BEGIN SEQUENCE block
 *  $SYNTAX$
 *      BREAK( <xExp> ) --> NIL
 *  $ARGUMENTS$
 *      <xExp> is any valid expression. It is allways required.
 *      If do not want to pass any argument, just use NIL.
 *  $RETURNS$
 *      NIL
 *  $DESCRIPTION$
 *      This function pass the control to the RECOVER statement in a
 *      BEGIN SEQUENCE block.
 *  $EXAMPLES$
 *      Break( NIL )
 *  $TESTS$
 *  $STATUS$
 *      R
 *  $COMPLIANCE$
 *      BREAK() is fully CA-Clipper compliant.
 *  $SEEALSO$
 *       BEGIN SEQUENCE 
 *  $END$
 */

/* NOTE: This function should normally have a parameter count check. But
         since in Harbour we cannot distinguish between BREAK() function and
         the BREAK statement, because both generate a BREAK() function
         call on the pcode level, we should drop the checking. [vszel] */

HARBOUR HB_BREAK( void )
{
   hb_vmRequestBreak( hb_param( 1, IT_ANY ) );
}
